﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using XNAWinformReversi.UI_Forms;

namespace XNAWinformReversi.GameLogic
{
    class AdvancedAI
    {
        enum Difficulty { Easy = 1, Intermediate, Expert };
        enum Piece { White = 1, Black };
        private int _difficulty;
        private int playerColor;
        private int nextPlayer;
        private int searchDepth;
        private int moveValue;
        private int score;
        private MainUIForm _form;
        private BoardModel _model;

        public AdvancedAI(int color, int diffictuly, BoardModel model)
        {
            playerColor = color;
            _difficulty = diffictuly;
            _model = model;
        }

        private void setDifficulty()
        {
            if (_difficulty == (int)Difficulty.Easy)
            {
                searchDepth = 3;
            }
            else if (_difficulty == (int)Difficulty.Intermediate)
            {
                searchDepth = 4;
            }
            else if (_difficulty == (int)Difficulty.Expert)
            {
                searchDepth = 7;
            }
        }

        private int changeColor(int color)
        {
            if (color == 1)
            {
                color = 2;
            }
            else
            {
                color = 1;
            }

            return color;
        }

        private bool checkGameOver(BoardModel board)
        {
            bool whiteCanMove = board.loadLegalMoves((int)Piece.White);
            bool blackCanMove = board.loadLegalMoves((int)Piece.Black);
            if (whiteCanMove == false && blackCanMove == false)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        public void getMove(MainUIForm form)
        {
            _form = form;
            setDifficulty();
            startGetBestMove(_model);
        }

        public void startGetBestMove(BoardModel board)
        {
            getBestMove(board, searchDepth, playerColor, int.MinValue, int.MaxValue);
        }

        public int getBestMove(BoardModel board, int depth, int currentPlayer, int alpha, int beta)
        {

            // If the game is over, terminate the search
            if (checkGameOver(board))
            {
                board.loadPieceCount();
                score = board.getScore();

                // if player has won
                if (score > 0)
                    moveValue = 100;
                // if player has lost (or tied)
                else
                    moveValue = -100;

                return moveValue;
            }

            // At the bottom of the search space
            if (depth == 0)
            {
                board.loadPieceCount();
                score = board.getScore();
                return moveValue;
            }

            // If this player can't play, skip turn
            if (board.loadLegalMoves(currentPlayer) == false)
            {
                nextPlayer = changeColor(currentPlayer);
                moveValue = getBestMove(board, depth, nextPlayer, alpha, beta);
                return moveValue;
            }

            // Get the list of plausable moves, sorted by most pieces flipped
            board.loadLegalMoves(currentPlayer);
            List<Square> moves = board.legalMovesList;

            // Find the best currentMove
            Square bestMove = Square.Empty;
            foreach (Square currentMove in moves)
            {
                // Copy the board
                BoardModel currentBoard = ObjectCopier.Clone(board);

                // Make the selected move
                currentBoard.setSquareValue(currentPlayer, currentMove.x, currentMove.y);
                currentBoard.loadPiecesToFlip(currentPlayer, currentMove.x, currentMove.y);

                // Evaluate the board
                nextPlayer = changeColor(currentPlayer);
                int currentValue = getBestMove(currentBoard, depth - 1, nextPlayer, alpha, beta);

                // MAX
                if (currentPlayer == playerColor)
                {
                    if (currentValue > alpha)
                    {
                        alpha = currentValue;
                        bestMove = currentMove;
                    }
                    // Prune
                    if (alpha >= beta)
                    {
                        return alpha;
                    }
                }
                // MIN
                else
                {
                    if (currentValue < beta)
                    {
                        beta = currentValue;
                        bestMove = currentMove;
                    }
                    // Prune
                    if (alpha >= beta)
                    {
                        return beta;
                    }
                }
            }

            // Make the best move
            if (!bestMove.Equals(Square.Empty))
                _form.startMove(currentPlayer, bestMove.x, bestMove.y);

            // Return the value of the best move
            if (currentPlayer == playerColor)
            {
                return alpha;
            }
            else
            {
                return beta;
            }
        }
    }
}
